#include<conio.h>
#include<dos.h>
#include<string.h>
#include<stdio.h>
#define _ERROR -1
#define TRUE	1
#define FALSE	0
#define BUF_SIZE 1024

/*	Codec-fixer by Royce Liao	03/97

	The stock S3 driver imposes a minimum horizontal stretch factor for
     video acceleration support in most video modes.  In other words,
	the codec-inifile disables Virge's video acceleration
	(YUV -> RGB + stretching) for playback-window size of 1X.

     This program alters the xxx.ini (5631_swc.ini, 8811_swc.ini, etc.)
     codec-inifile which comes with S3's Virge Win95 drivers.

     The code scans the user specified input-file for 4-digit numbers of
     the form "XY00" (last two digits must be '0'.), replaces XY with 10,
     so the final 4-digit number = "1000", which corresponds to a h-stretch
     factor of exactly 1.0X

*/

extern int _argc;
extern char **_argv;


/* Locate "=" in input text string
	SUCCESS Returns position in string [ 0..n-1 ]
	FAILURE Returns -1	*/
int
find_equal( const char *text )
{
	int i=0, marker= _ERROR;
     while ( text != NULL && text[ i ] != NULL )
     {
     	if ( text[ i ] == '=' )
          	marker = i;
          ++i;
	}
     return marker;
}


int
find_words( char *words[], const char *text )
{
	int count=0;
     int i=0;

     if ( text == NULL )
     	return _ERROR;	/* No string to search */

     while( text[ i ] != NULL )
     {
          while ( text[ i ] != NULL && isspace( text[ i ]) )
          	++i;
          if ( text[ i ] != NULL )
          {
          	words[ count ] = (char*)&text[ i ];
               ++count;
          }
          while ( text[ i ] != NULL && !isspace( text[ i ]) )
          	++i;
     }
     return count;

}


void
fix_4value( char *text )
{
	int i=0;
	if ( text == NULL )
     	return;

	/* Check length , must be 4! */

     while ( text[ i ] != NULL && !isspace( text[ i ] ) )
     	++i;

     if ( i != 4 )
     	return;

     if ( text[ 2 ] == '0' && text[ 3 ] == '0' )
          if ( isdigit( text[ 0 ] ) && isdigit( text[ 1 ] ) )
         	{
	     	text[ 0 ] = '1';
     	     text[ 1 ] = '0';
          }

	return;
}


void
help_screen( void )
{
	printf("\nUsage : fixup.exe [source file] {output file}" );
     printf("\n\n\t[source file] -- (REQUIRED) specifies input file " );
     printf("\n\t{output file} -- (OPTIONAL) specifies output file, default");
     printf(" = FIXED.INI ");
     printf("\n\n\tFIXUP.EXE adjusts the S3 Trio64V+/Virge series codec-ini");
	printf("\n\tfiles by replacing 'XX00' with '1000'.  The S3's DDraw HAL");
	printf("\n\tuses the codec-ini files to determine the availability");
	printf("\n\tof the S3's hardware video-acceleration overlay.");
     printf("\n\t(Run FIXUP.EXE on any XXX_SWC.INI file in your \WINDOWS dir.");
     printf("\n\t Examples : 8811_SWC.INI 883D_SWC.INI 5631_SWC.INI 8814_SWC.INI )");
	printf("\n\n\tPlease note that the stock codec-ini files are optimized");
	printf("\n\tfor standard RAM-timings (50MHz 1-cycle EDO, 60MHz 2-cycle.)");
	printf("\n\n\tSmaller playback windows require more aggressive (faster)");
	printf("\n\tdisplay-RAM timings.  Otherwise, display-anomalies will occur");
	printf("\n\tduring overlayed-video playback.");
	return;
}

void
fixup( FILE *in, FILE *out )
{
     char buffer[ BUF_SIZE ], *status, *words[ BUF_SIZE ];
     int line=0, eq, num_words, i, past_middle = FALSE;

     do	{
		status = fgets( buffer, BUF_SIZE, in );
          eq = find_equal( buffer );
          if ( eq != _ERROR && past_middle==FALSE )
          {
          	num_words=find_words( words, &buffer[ eq + 1 ] );
               printf(" %d", num_words );

               for( i=0; i < num_words; ++i )
               	fix_4value( words[ i ] );
		}

          /* Halts the avove replacement routine
             when the 2nd-half of the file is encountered */
		if ( line > 10 && buffer != NULL )
          	if ( buffer[ 0 ] == '[' )
               	past_middle= TRUE;

          fputs( buffer, out );
          ++line;
     }	while ( status != NULL );
}


void
main( void )
{
	char srcname[ 50 ], outname[ 50 ] = "fixed.ini";
	FILE *fpin, *fpout;
     char *words[ 255 ], ch ;
     int num_words, n;

     if ( _argc < 2 )	{
     	help_screen();
          exit();
     }

     strcpy( srcname, _argv[ 1 ] );
     fpin = fopen( srcname, "rt" );

     if ( fpin == NULL )	{
     	fprintf( stderr, "\nError!  Could not read %s!", srcname );
          exit();
     }


     if ( _argc > 2 && _argv[ 2 ] != NULL )
     	strcpy( outname, _argv[ 2 ] );


     fpout = fopen( outname, "rt" );
     if ( fpout != NULL )	{
     	printf("\nWarning, %s already exists.  Overwrite (y/n) ?",
			outname );
          scanf("%c", &ch );
     	if ( ch != 'y' && ch != 'Y' )	{
          	printf( "\nAborted execution." );
               fclose( fpin );
               exit();
          }
          fclose( fpout );
     }

     fpout = fopen( outname, "wt" );

     if ( fpout == NULL )	{
     	fprintf( stderr, "\nError!  Could not write to %s!", outname );
          exit();
     }

     fixup( fpin, fpout );

     fclose( fpin );
     fclose( fpout );
     printf("\nProcessed data written to %s", outname );
}